@@ -1,14 +1,6 @@
-"""
-
- 2022 (c) piteren
-
-    RL Actor
-
-"""
-
 from abc import abstractmethod, ABC
 import numpy as np
-from typing import Optional
+from typing import Optional, Dict, Any, List
 
 from pypaq.lipytools.printout import stamp
 from pypaq.lipytools.pylogger import get_pylogger
@@ -23,7 +15,7 @@ class Actor(ABC):
     @abstractmethod
     def get_policy_action(self, observation: object) -> object: pass
 
-# cooperates with RLTrainer, prepares obs_vec, updates, saves
+# cooperates with RLTrainer, prepares obs_vec, updates self policy, saves
 class TrainableActor(Actor, ABC):
 
     def __init__(
@@ -48,16 +40,31 @@ class TrainableActor(Actor, ABC):
 
     # prepares numpy vector from observation, first tries to get from RLEnvy
     def _get_observation_vec(self, observation: object) -> np.ndarray:
-        try: return self._envy.prep_observation_vec(observation)
-        except RLException: raise RLException ('TrainableActor not implemented _get_observation_vec()')
+        try:
+            return self._envy.prep_observation_vec(observation)
+        except RLException:
+            raise RLException ('TrainableActor should implement _get_observation_vec()')
 
-    # add sampling (from probability?) option which may be helpful for training
+    # adds sampling (from probability?) option which may be helpful for training
     @abstractmethod
     def get_policy_action(self, observation:object, sampled=False) -> object: pass
 
-    # updates self with (batch of) experience data (given with kwargs), returns dict with Actor "metrics" - loss etc.
+    # updates policy
     @abstractmethod
-    def update_with_experience(self, **kwargs) -> dict: pass
+    def update_with_experience(
+            self,
+            batch: List[Dict[str,Any]],
+            inspect: bool,
+    ) -> Dict[str,Any]:
+        """
+        updates (self) policy with batch of experience data given with kwargs
+        returns dict with some update "metrics" like a loss etc.
+        those metrics may be used by RLTrainer for publish, monitoring, training process update..
+        currently supported:
+        - any number (float?) will be published to TB (with key from dict and Trainer._upd_step)
+        - 'zeroes' will be published by Trainer._zepro
+        """
+        pass
 
     @abstractmethod
     # returns Actor TOP save directory
@@ -67,7 +74,7 @@ class TrainableActor(Actor, ABC):
     def get_save_dir(self) -> str:
         return f'{self._get_save_topdir()}/{self.name}'
 
-    # saves (self) Actor state
+    # saves Actor (self)
     @abstractmethod
     def save(self): pass
 
