@@ -1,62 +1,60 @@
-"""
-
- 2021 (c) piteren
-
-    PG_Actor - PolicyGradients TrainableActor, NN based
-
-    TODO: implement parallel training, in batches (many envys)
-
-"""
-
-from abc import ABC, abstractmethod
+from abc import ABC
 import numpy as np
-from typing import Optional, Callable, Union, List
+from typing import Optional, List, Dict, Any
 
+from pypaq.R4C.helpers import extract_from_batch
 from pypaq.R4C.actor import TrainableActor
 from pypaq.R4C.envy import FiniteActionsRLEnvy
-from pypaq.torchness.comoneural import NNWrap
+from pypaq.R4C.policy_gradients.pg_actor_module import PGActorModule
+from pypaq.torchness.motorch import MOTorch, Module
 
+# TODO: implement parallel training, in batches (many envys)
 
+# PolicyGradients TrainableActor, NN based
 class PGActor(TrainableActor, ABC):
 
     def __init__(
             self,
             envy: FiniteActionsRLEnvy,
-            nnwrap: type(NNWrap),
-            module_type: Optional[Union[Callable, type]]=   None,
-            seed: int=                                      123,
+            name: str=                              'PGActor',
+            module_type: Optional[type(Module)]=    PGActorModule,
+            seed: int=                              123,
             **kwargs):
 
         TrainableActor.__init__(
             self,
             envy=   envy,
+            name=   name,
             **kwargs)
         self._envy = envy  # to update type (for pycharm only)
 
         np.random.seed(seed)
 
         # some overrides and updates
-        kwargs['name'] = self.name
-        kwargs['name_timestamp'] = False                # name timestamp is driven by TrainableActor (with self.name)
         if 'logger' in kwargs: kwargs.pop('logger')     # NNWrap will always create own logger (since then it is not given) with optionally given level
         kwargs['num_actions'] = self._envy.num_actions()
         kwargs['observation_width'] = self._get_observation_vec(self._envy.get_observation()).shape[-1]
 
-        self.nnw: NNWrap = nnwrap(module_type=module_type, seed=seed, **kwargs)
+        self.model = MOTorch(
+            module_type=    module_type,
+            name=           self.name,
+            seed=           seed,
+            **kwargs)
 
-        self._rlog.info('*** PG_Actor *** (NN based) initialized')
-        self._rlog.info(f'> NNWrap: {nnwrap.__name__}')
+        self._rlog.info(f'*** PGActor : {self.name} *** (NN based) initialized')
 
     # vectorization of observations batch, may be overridden with more optimal custom implementation
     def _get_observation_vec_batch(self, observations: List[object]) -> np.ndarray:
         return np.asarray([self._get_observation_vec(v) for v in observations])
 
-    @abstractmethod
-    def get_policy_probs(self, observation: object) -> np.ndarray: pass
+    def get_policy_probs(self, observation: object) -> np.ndarray:
+        obs_vec = self._get_observation_vec(observation)
+        return self.model(obs_vec)['probs'].detach().cpu().numpy()
 
-    # baseline, may be overridden with more optimal custom implementation
+    # batch call to NN
     def get_policy_probs_batch(self, observations: List[object]) -> np.ndarray:
-        return np.asarray([self.get_policy_probs(o) for o in observations])
+        obs_vecs = self._get_observation_vec_batch(observations)
+        return self.model(obs_vecs)['probs'].detach().cpu().numpy()
 
     # gets policy action based on policy (action) probs
     def get_policy_action(self, observation: object, sampled=False) -> int:
@@ -71,11 +69,31 @@ class PGActor(TrainableActor, ABC):
         else:       actions = np.argmax(probs, axis=-1)
         return actions
 
+    # updates self NN with batch of data
+    def update_with_experience(
+            self,
+            batch: List[Dict[str,Any]],
+            inspect: bool,
+    ) -> Dict[str, Any]:
+
+        observations = extract_from_batch(batch, 'observation')
+
+        obs_vecs = self._get_observation_vec_batch(observations)
+        out = self.model.backward(
+            observation=    obs_vecs,
+            action_taken=   extract_from_batch(batch, 'action'),
+            dreturn=        extract_from_batch(batch, 'dreturn'))
+
+        out.pop('logits')
+        if 'probs' in out: out['probs'] = out['probs'].cpu().detach().numpy()
+
+        return out
+
     def _get_save_topdir(self) -> str:
-        return self.nnw['save_topdir']
+        return self.model['save_topdir']
 
     def save(self):
-        self.nnw.save()
+        self.model.save()
 
     def __str__(self):
-        return self.nnw.__str__()
\ No newline at end of file
+        return self.model.__str__()
\ No newline at end of file
